@@ -7,12 +7,12 @@
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation; either version 3 of the License, or
 #   (at your option) any later version.
-# 
+#
 #   This program is distributed in the hope that it will be useful,
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #   GNU General Public License for more details.
-# 
+#
 #   You should have received a copy of the GNU General Public License along
 #   with this program.  If not, see <https://www.gnu.org/licenses/>.
 
@@ -22,16 +22,28 @@
 # https://docs.python.org/3/whatsnew/3.0.html#pep-3101-a-new-approach-to-string-formatting
 
 ###### Standard library imports ######
-import datetime     #from datetime import datetime, timedelta
-import math
-import sys			# required for .stdout.write()
+# don't confuse the 'date' method with the 'Date' variable!
+from datetime import date, datetime, timedelta
+from math import copysign
 
 ###### Local application imports ######
-from sfalmanac.lib import config as config
+from sfalmanac.lib import config
 from sfalmanac.lib.ld_skyfield import getDUT1, moon_GHA, moon_SD, moon_VD, ld_planets, ld_stars, find_transit, sunSD
 
 UpperLists = [[], [], []]    # moon GHA per hour for 3 days
 
+#------------------------
+#   internal functions
+#------------------------
+
+def fmtdate(d):
+    if config.pgsz == 'Letter': return d.strftime("%m/%d/%Y")
+    return d.strftime("%d.%m.%Y")
+
+def fmtdates(d1,d2):
+    if config.pgsz == 'Letter': return d1.strftime("%m/%d/%Y") + " - " + d2.strftime("%m/%d/%Y")
+    return d1.strftime("%d.%m.%Y") + " - " + d2.strftime("%d.%m.%Y")
+
 def buildUPlists(n, ghaSoD, ghaPerHour, ghaEoD):
     # build list of hourly GHA values with modified start and end time to
     #  account for rounding times to the minute where 23:59:>30 rounds up
@@ -63,26 +75,30 @@ def NSdeg(deg, modern=False, hr=0, forceNS=False):
     return sdeg, hemisph
 
 
-def moontab(date, strat, dop):
+#---------------------------------
+#   create Lunar Distance table
+#---------------------------------
+
+def moontab(Date, dpp, strat):
     # generates LaTeX table for moon and Lunar Distance (traditional style)
 
     tab = r'''\setlength{\tabcolsep}{5pt}  % default 6pt
 \noindent'''
     n = 0
-    while n < dop:      # maximum 3 days on a page
+    while n < dpp:      # maximum 3 days on a page
 
 # >>>>>>>>>>>> Calculate all required data <<<<<<<<<<<<
 
         if config.debug_strategy:
             print("=" * 70)
-        date0 = date - datetime.timedelta(days=1)
-        gham, decm, degm, HPm, GHAupper, GHAlower, ghaSoD, ghaEoD = moon_GHA(date)
-        vmin, dmin = moon_VD(date0,date)
+        Date0 = Date - timedelta(days=1)
+        gham, decm, degm, HPm, GHAupper, GHAlower, ghaSoD, ghaEoD = moon_GHA(Date)
+        vmin, dmin = moon_VD(Date0,Date)
 
         buildUPlists(n, ghaSoD, GHAupper, ghaEoD)
 
-        out2, tup2, NMhours, ra_m = ld_planets(date)   # planets & sun
-        out, tup = ld_stars(date, NMhours, out2[0][1].hours)
+        out2, tup2, NMhours, ra_m = ld_planets(Date)   # planets & sun
+        out, tup = ld_stars(Date, NMhours, out2[0][1].hours)
         tup = tup + tup2
         tup.sort(key = lambda x: x[1])  # sort by signed first valid LD
         if config.debug_strategy:
@@ -103,8 +119,8 @@ def moontab(date, strat, dop):
             # build list of objects sorted by largest hourly LD delta first
             tuple_list = [None] * 27
             for i in range(len(tup)):
-##                tuple_list[i] = (tup[i][0], tup[i][4], math.copysign(1, tup[i][1]), tup[i][3])
-                tuple_list[i] = (tup[i][0], tup[i][5], math.copysign(1, tup[i][1]), tup[i][4])
+##                tuple_list[i] = (tup[i][0], tup[i][4], copysign(1, tup[i][1]), tup[i][3])
+                tuple_list[i] = (tup[i][0], tup[i][5], copysign(1, tup[i][1]), tup[i][4])
             tuple_list.sort(key = lambda x: x[1])   # sort by object magnitude
             if config.debug_strategy:
                 print("--- tuples with highest brightness first ---")
@@ -122,8 +138,8 @@ def moontab(date, strat, dop):
             # build list of objects sorted by largest hourly LD delta first
             tuple_list = [None] * 27
             for i in range(len(tup)):
-##                tuple_list[i] = (tup[i][0], tup[i][2], math.copysign(1, tup[i][1]), tup[i][3])
-                tuple_list[i] = (tup[i][0], tup[i][3], math.copysign(1, tup[i][1]), tup[i][4])
+##                tuple_list[i] = (tup[i][0], tup[i][2], copysign(1, tup[i][1]), tup[i][3])
+                tuple_list[i] = (tup[i][0], tup[i][3], copysign(1, tup[i][1]), tup[i][4])
             tuple_list.sort(key = lambda x: -x[1])  # sort by max hourly LD delta
             if config.debug_strategy:
                 print("--- tuples with largest ld_delta_max first ---")
@@ -270,7 +286,7 @@ def moontab(date, strat, dop):
             iC += 1
 
         if sunSDrqrd:
-            sdsm = sunSD(date)  # get sun's SD at 0h and 23h
+            sdsm = sunSD(Date)  # get sun's SD at 0h and 23h
             ldx00 = ld[iC][0]
             ldx23 = ld[iC][23]
             if ldx00.find("circ") == -1: ldx00 = ''
@@ -304,7 +320,7 @@ def moontab(date, strat, dop):
 
         tab = tab + r'''\\
 \hline
-\multicolumn{{1}}{{|c|}}{{\rule{{0pt}}{{2.6ex}}\textbf{{{}}}}} & \multicolumn{{1}}{{c}}{{\textbf{{GHA}}}} & \multicolumn{{1}}{{c}}{{\(\nu\)}} & \multicolumn{{1}}{{c}}{{\textbf{{Dec}}}} & \multicolumn{{1}}{{c}}{{\textit{{d}}}} & \multicolumn{{1}}{{c|}}{{\textbf{{HP}}}}'''.format(date.strftime("%a"))
+\multicolumn{{1}}{{|c|}}{{\rule{{0pt}}{{2.6ex}}\textbf{{{}}}}} & \multicolumn{{1}}{{c}}{{\textbf{{GHA}}}} & \multicolumn{{1}}{{c}}{{\(\nu\)}} & \multicolumn{{1}}{{c}}{{\textbf{{Dec}}}} & \multicolumn{{1}}{{c}}{{\textit{{d}}}} & \multicolumn{{1}}{{c|}}{{\textbf{{HP}}}}'''.format(Date.strftime("%a"))
 
         for iC in range(iCols):
             tab = tab + r''' & \multicolumn{{1}}{{c|}}{{\textbf{{{}}}}}'''.format(obj[iC])
@@ -327,7 +343,7 @@ def moontab(date, strat, dop):
                 nextDECm = degm[23]	    # hour 24 = hour 23
 
             mdec, mNS = NSdeg(decm[h],False,h)
-            if mNS != mlastNS or math.copysign(1.0,prevDECm) != math.copysign(1.0,nextDECm):
+            if mNS != mlastNS or copysign(1.0,prevDECm) != copysign(1.0,nextDECm):
                 mdec, mNS = NSdeg(decm[h],False,h,True)	# force N/S
             mlastNS = mNS
 
@@ -355,8 +371,8 @@ def moontab(date, strat, dop):
             tab = tab + line + lineterminator
             h += 1
 
-        sdmm = moon_SD(date)
-        mp_upper = find_transit(date, UpperLists[n], False)    # calculate moon upper transit
+        sdmm = moon_SD(Date)
+        mp_upper = find_transit(Date, UpperLists[n], False)    # calculate moon upper transit
         tab = tab + r'''\hline
 \rule{{0pt}}{{2.4ex}} & \multicolumn{{5}}{{c|}}{{SD = {}$'$ \quad Mer. pass. {}}}'''.format(sdmm,mp_upper)
         if iCols > 0:
@@ -374,7 +390,7 @@ def moontab(date, strat, dop):
             tab = tab + r'''\multicolumn{5}{c}{}\\[-1.5ex]
 '''
         n += 1
-        date += datetime.timedelta(days=1)
+        Date += timedelta(days=1)
         tab = tab + r'''\end{tabular}
 \par\noindent    % put next table below here'''
     return tab
@@ -383,198 +399,108 @@ def moontab(date, strat, dop):
 #   page preparation
 #----------------------
 
-def page(date, strat, dop):
+def page(Date, dpp, strat):
     # creates a page (max. 3 days) of tables
 
     # time delta values for the initial date&time...
-    dut1, deltat = getDUT1(date)
+    dut1, deltat = getDUT1(Date)
     timeDUT1 = r"DUT1 = UT1-UTC = {:+.4f} sec\quad$\Delta$T = TT-UT1 = {:+.4f} sec".format(dut1, deltat)
 
+    Lfoot_IERSEOP = ""
+    if config.dt_IERSEOP != None:
+        # the IERS EOP data start date is 2nd January 1973
+        if Date + timedelta(days=2) >= date(1973, 1, 2):
+            Lfoot_IERSEOP = config.txtIERSEOP
+        if Date + timedelta(days=2) >= config.dt_IERSEOP:
+            Lfoot_IERSEOP = config.endIERSEOP
+        if Date > config.dt_IERSEOP:
+            Lfoot_IERSEOP = r'''\textbf{No IERS EOP prediction data available}'''
+
     page = ''
 
-    if dop > 1:
+    if dpp > 1:
         str2 = r'''\textbf{{{} to {} UT}}
-'''.format(date.strftime("%Y %B %d"),(date+datetime.timedelta(days=dop-1)).strftime("%b. %d"))
+'''.format(Date.strftime("%Y %B %d"),(Date+timedelta(days=dpp-1)).strftime("%b. %d"))
     else:
         str2 = r'''\textbf{{{} UT}}
-'''.format(date.strftime("%Y %B %d"))
+'''.format(Date.strftime("%Y %B %d"))
 
-    str1 = r'''
+    if config.FANCYhd:
+        str1 = r'''
 % ------------------ N E W   P A G E ------------------
 \newpage
 \sffamily
-\lhead{{\noindent\textsf{{\footnotesize{{{}}}}}}}
+\lhead{{\textsf{{\footnotesize{{{}}}}}}}
 \rhead{{\textsf{{\textbf{{{}}}}}}}
+\lfoot{{\textsf{{\footnotesize{{{}}}}}}}
+\begin{{scriptsize}}
+'''.format(timeDUT1, str2, Lfoot_IERSEOP)
+    else:   # old formatting
+        str1 = r'''
+% ------------------ N E W   P A G E ------------------
+\newpage
+\sffamily
+\noindent
+\begin{{flushleft}}     % required so that \par works
+{{\footnotesize {}}}\hfill{}
+\end{{flushleft}}\par
 \begin{{scriptsize}}
 '''.format(timeDUT1, str2)
 
     page = page + str1
 
-    page = page + moontab(date,strat,dop)
+    page = page + moontab(Date,dpp,strat)
     page = page + r'''
 \end{scriptsize}'''
     # to avoid "Overfull \hbox" messages, leave a paragraph end before the end of a size change. (See lines above)
     return page
 
 
-def pages(first_day, strat, daysnum):
+def pages(first_day, dtp, strat):
     # make pages beginning with first_day
+    # dtp = 0 if for entire year; = -1 if for entire month; else days to print
+
     out = ''
     pmth = ''
+    dpp = 3         # 3 days per page maximum
+    day1 = first_day
+
+    if dtp == 0:        # if entire year
+        year = first_day.year
+        yr = year
+        while year == yr:
+            day3 = day1 + timedelta(days=2)
+            if day3.year != yr:
+                dpp -= day3.day
+                if dpp <= 0: return out
+            out += page(day1,dpp,strat)
+            day1 += timedelta(days=3)
+            year = day1.year
+
+    elif dtp == -1:     # if entire month
+        mth = first_day.month
+        m = mth
+        while mth == m:
+            day3 = day1 + timedelta(days=2)
+            if day3.month != m:
+                dpp -= day3.day
+                if dpp <= 0: return out
+            out += page(day1,dpp,strat)
+            day1 += timedelta(days=3)
+            mth = day1.month
+
+    else:           # print 'dtp' days beginning with first_day
+        i = dtp   # don't decrement dtp
+        while i > 0:
+            if i < 3: dpp = i
+            out += page(day1,dpp,strat)
+            i -= 3
+            day1 += timedelta(days=3)
 
-    while daysnum > 0:
-        daysnum -= 3
-        dop = 3 if daysnum >= 0 else daysnum+3      # days to print on page
-        out += page(first_day,strat,dop)
-        first_day += datetime.timedelta(days=3)
     return out
 
-#--------------------------
-#   external entry point
-#--------------------------
-
-def makeLDtables(first_day, strat, daysnum, entireMth, entireYr, spad, df180, df360):
-
-    # make tables starting from first_day
-    year = first_day.year
-    mth = first_day.month
-    day = first_day.day
-
-    # page size specific parameters
-    # NOTE: 'bm' (bottom margin) is an unrealistic value used only to determine the vertical size of 'body' (textheight), which must be large enough to include all the tables. 'tm' (top margin) and 'hs' (headsep) determine the top of body. Finally use 'fs' (footskip) to position the footer.
-    if config.pgsz == "A4":
-        # A4 ... pay attention to the limited page width
-        paper = "a4paper"
-        # title page...
-        vsep1 = "1.5cm"
-        vsep2 = "1.0cm"
-        tm1 = "21mm"
-        bm1 = "15mm"
-        lm1 = "10mm"
-        rm1 = "10mm"
-        # description text...
-        tm2 = "21mm"
-        bm2 = "18mm"
-        # data pages...
-        tm = "34.5mm"       # was "21mm" [v2q]
-        bm = "12mm"         # was "18mm" [v2q]
-        hs = "5.1pt"        # headsep  (page 3 onwards) [v2q]
-        fs = "24pt"         # footskip (page 3 onwards) [v2q]
-        lm = "16mm"
-        rm = "12mm"
-    else:
-        # LETTER ... pay attention to the limited page height
-        paper = "letterpaper"
-        # title page...
-        vsep1 = "0.8cm"
-        vsep2 = "0.7cm"
-        tm1 = "12mm"
-        bm1 = "15mm"
-        lm1 = "12mm"
-        rm1 = "12mm"
-        # description text...
-        tm2 = "12mm"
-        bm2 = "13mm"
-        # data pages...
-        tm = "25.6mm"       # was "12mm" [v2q]
-        bm = "5mm"          # was "12mm" [v2q]
-        hs = "2.6pt"        # headsep  (page 3 onwards) [v2q]
-        fs = "21pt"         # footskip (page 3 onwards) [v2q]
-        lm = "16mm"
-        rm = "14mm"
-
-    # default is 'oneside'...
-    alm = r'''\documentclass[10pt, {}]{{report}}'''.format(paper)
-
-    alm = alm + r'''
-%\usepackage[utf8]{inputenc}
-\usepackage[english]{babel}
-\usepackage{fontenc}
-\usepackage{enumitem} % used to customize the {description} environment'''
-
-    # to troubleshoot add "showframe, verbose," below:
-    alm = alm + r'''
-\usepackage[nomarginpar, top={}, bottom={}, left={}, right={}]{{geometry}}
-\setlength{{\headsep}}{{{}}} % [v2q]'''.format(tm,bm,lm,rm,hs)
-
-    # Note: \DeclareUnicodeCharacter is not compatible with some versions of pdflatex
-    alm = alm + r'''
-\usepackage{xcolor}  % highlight double moon events on same day
-\definecolor{khaki}{rgb}{0.76, 0.69, 0.57}
-\usepackage{multirow}
-\newcommand{\HRule}{\rule{\linewidth}{0.5mm}}
-\usepackage{fancyhdr}   % [v2q]
-\pagestyle{fancy}       % [v2q]
-\renewcommand{\headrulewidth}{0pt}  % [v2q]
-\setlength{\footskip}{20pt}
-\usepackage[pdftex]{graphicx}	% for \includegraphics
-\usepackage{tikz}				% for \draw  (load after 'graphicx')
-%\showboxbreadth=50  % use for logging
-%\showboxdepth=50    % use for logging
-%\DeclareUnicodeCharacter{00B0}{\ensuremath{{}^\circ}}
-\setlength\fboxsep{1.5pt}       % ONLY used by \colorbox in ldist_skyfield.py
-\begin{document}'''
-
-    alm = alm + r'''
-% for the title page only...
-\newgeometry{{nomarginpar, top={}, bottom={}, left={}, right={}}}'''.format(tm1,bm1,lm1,rm1)
-
-    alm = alm + r'''
-    \begin{titlepage}
-    \begin{center}
-    \textsc{\Large Generated using Skyfield}\\
-    \large http://rhodesmill.org/skyfield/\\[0.7cm]'''
-
-    if config.dockerized:   # DOCKER ONLY
-        df180 = "../A4chart0-180_P.pdf"
-        df360 = "../A4chart180-360_P.pdf"
-
-    alm = alm + r'''
-    % TRIM values: left bottom right top
-    \includegraphics[clip, trim=12mm 20cm 12mm 21mm, width=0.92\textwidth]{{{}}}\\[0.3cm]
-    \includegraphics[clip, trim=12mm 20cm 12mm 21mm, width=0.92\textwidth]{{{}}}\\'''.format(df180,df360)
-    
-    alm = alm + r'''[{}]
-    \textsc{{\huge Lunar Distance}}\\[{}]'''.format(vsep1,vsep2)
-
-    if entireYr:
-        alm = alm + r'''
-    \HRule \\[0.5cm]
-    {{ \Huge \bfseries {}}}\\[0.2cm]
-    \HRule \\'''.format(year)
-    elif entireMth:
-        alm = alm + r'''
-    \HRule \\[0.5cm]
-    {{ \Huge \bfseries {}}}\\[0.2cm]
-    \HRule \\'''.format(first_day.strftime("%B %Y"))
-    else:
-        alm = alm + r'''
-    \HRule \\[0.5cm]
-    {{ \Huge \bfseries from {}.{}.{}}}\\[0.2cm]
-    \HRule \\'''.format(day,mth,year)
-
-    alm = alm + r'''
-    \begin{center}\begin{tabular}[t]{rl}
-    \large\emph{Author:} & \large Andrew \textsc{Bauer}\\
-    \end{tabular}\end{center}'''
-
-    alm = alm + r'''
-    {\large \today}
-    \HRule \\[0.2cm]
-    \end{center}
-    \begin{description}[leftmargin=5.5em,style=nextline]\footnotesize
-    \item[Disclaimer:] These are computer generated tables - use them at your own risk. They replicate Lunar Distance algorithms with no guarantee of accuracy. They are intended to encourage the use of sextants, be it as a hobby or as a backup when electronics fail. The author claims no liability for any consequences arising from use of these tables and accompanying charts.
-    \end{description}
-\end{titlepage}'''
-
-    alm = alm + r'''
-% for the description text only...
-\newgeometry{{nomarginpar, top={}, bottom={}, left={}, right={}}}'''.format(tm2,bm2,lm,rm)
-
-    alm = alm + r'''
-\lfoot{\textsf{\footnotesize{https://thenauticalalmanac.com/}}}
-\cfoot{\centerline{Page \thepage}}
-\rfoot{\textsf{\footnotesize{https://pypi.org/project/sfalmanac/}}}
+def page2():
+    return r'''
 \setcounter{page}{2}    % otherwise it's 1
     %\vspace*{5mm}
     \noindent
@@ -618,14 +544,333 @@ def makeLDtables(first_day, strat, daysnum, entireMth, entireYr, spad, df180, df
     ``The rate of change of LD becomes zero when LD passes through a minimum or maximum, making an observation useless.''\footnote{Henning Umland, Chapter 7 - Finding Time and Longitude by Lunar Distances}
     \item the rate of change of the hourly LD delta does not exceed 0.016$^\circ$ (= 0.96$'$). This empirical figure removes LD values where linear interpolation (between hours) becomes unreliable.
     \end{itemize}
-    Suggested further reading: ``Stark Tables: For Clearing the Lunar Distance and Finding Universal Time by Sextant Observation'' by Bruce Stark, ISBN 978-0-914025-21-4
+    Suggested further reading: ``Stark Tables: For Clearing the Lunar Distance and Finding Universal Time by Sextant Observation'' by Bruce Stark, ISBN 978-0-914025-21-4'''
+
+#--------------------------
+#   external entry point
+#--------------------------
+
+def makeLDtables(first_day, dtp, strat, df180, df360):
+
+    if config.FANCYhd:
+        return makeLDnew(first_day, dtp, strat, df180, df360) # use the 'fancyhdr' package
+    else:
+        return makeLDold(first_day, dtp, strat, df180, df360) # use old formatting
+
+#   The following functions are intentionally separate functions.
+#   'makeEVold' is required for TeX Live 2019, which is the standard
+#   version in Ubuntu 20.04 LTS which expires in April 2030.
+
+def hdrEVnew(first_day,dtp,vsep1,vsep2,df180,df360):
+    # build the front page
+    year = first_day.year
+    mth = first_day.month
+    day = first_day.day
+
+    tex = r'''
+\pagestyle{frontpage}
+    \begin{titlepage}
+    \begin{center}
+    \textsc{\Large Generated using Skyfield}\\
+    \large http://rhodesmill.org/skyfield/\\[0.7cm]'''
+
+    if config.dockerized:   # DOCKER ONLY
+        df180 = "../A4chart0-180_P.pdf"
+        df360 = "../A4chart180-360_P.pdf"
+
+    tex += r'''
+    % TRIM values: left bottom right top
+    \includegraphics[clip, trim=12mm 20cm 12mm 21mm, width=0.92\textwidth]{{{}}}\\[0.3cm]
+    \includegraphics[clip, trim=12mm 20cm 12mm 21mm, width=0.92\textwidth]{{{}}}\\'''.format(df180,df360)
+    
+    tex += r'''[{}]
+    \textsc{{\huge Lunar Distance}}\\[{}]'''.format(vsep1,vsep2)
+
+    if dtp == 0:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(first_day.year)
+    elif dtp == -1:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(first_day.strftime("%B %Y"))
+    elif dtp > 1:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(fmtdates(first_day,first_day+timedelta(days=dtp-1)))
+    else:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(fmtdate(first_day))
+
+    tex += r'''
+    \begin{center}\begin{tabular}[t]{rl}
+    \large\emph{Author:} & \large Andrew \textsc{Bauer}\\
+    \end{tabular}\end{center}'''
+
+    tex += r'''
+    {\large \today}
+    \HRule \\[0.2cm]
+    \end{center}
+    \begin{description}[leftmargin=5.5em,style=nextline]\footnotesize
+    \item[Disclaimer:] These are computer generated tables - use them at your own risk. They replicate Lunar Distance algorithms with no guarantee of accuracy. They are intended to encourage the use of sextants, be it as a hobby or as a backup when electronics fail. The author claims no liability for any consequences arising from use of these tables and accompanying charts.
+    \end{description}
+\end{titlepage}
+\pagestyle{page2}  % page style for page 2'''
+
+    tex += page2()
+
+    return tex
+
+def makeLDnew(first_day, dtp, strat, df180, df360):
+    # make tables starting from first_day
+
+    # page size specific parameters
+    # NOTE: 'bm' (bottom margin) is an unrealistic value used only to determine the vertical size of 'body' (textheight), which must be large enough to include all the tables. 'tm' (top margin) and 'hs' (headsep) determine the top of body. Finally use 'fs' (footskip) to position the footer.
+    if config.pgsz == "A4":
+        # A4 ... pay attention to the limited page width
+        paper = "a4paper"
+        # title page...
+        vsep1 = "1.5cm"
+        vsep2 = "1.0cm"
+        tm1 = "21mm"
+        bm1 = "15mm"
+        lm1 = "10mm"
+        rm1 = "10mm"
+        # page 2 text...
+        tm2 = "21mm"
+        bm2 = "18mm"
+        fs2 = "20pt"
+        # data pages...
+        tm = "27.6mm"       # was "21mm"
+        bm = "16mm"         # was "18mm"
+        hs = "4.9pt"        # headsep  (page 3 onwards)
+        fs = "12pt"         # footskip (page 3 onwards)
+        lm = "16mm"
+        rm = "12mm"
+    else:
+        # LETTER ... pay attention to the limited page height
+        paper = "letterpaper"
+        # title page...
+        vsep1 = "0.8cm"
+        vsep2 = "0.7cm"
+        tm1 = "12mm"
+        bm1 = "15mm"
+        lm1 = "12mm"
+        rm1 = "12mm"
+        # page 2 text...
+        tm2 = "12mm"
+        bm2 = "13mm"
+        fs2 = "20pt"
+        # data pages...
+        tm = "17.8mm"       # was "12mm"
+        bm = "10.5mm"       # was "12mm"
+        hs = "2.6pt"        # headsep  (page 3 onwards)
+        fs = "12pt"         # footskip (page 3 onwards)
+        lm = "16mm"
+        rm = "14mm"
+
+    # default is 'oneside'...
+    tex = r'''\documentclass[10pt, {}]{{report}}'''.format(paper)
+
+    tex += r'''
+%\usepackage[utf8]{inputenc}
+\usepackage[english]{babel}
+\usepackage{fontenc}
+\usepackage{enumitem} % used to customize the {description} environment'''
+
+    # to troubleshoot add "showframe, verbose," below:
+    tex += r'''
+\usepackage[nomarginpar, top={}, bottom={}, left={}, right={}]{{geometry}}'''.format(tm1,bm1,lm1,rm1)
+
+    # define page styles
+    # CAUTION: putting '\fancyhf{}' in frontpage style also clears the footer in page2!
+    tex += r'''
+%------------ page styles ------------
+\usepackage{fancyhdr}
+\renewcommand{\headrulewidth}{0pt}
+\renewcommand{\footrulewidth}{0pt}
+\fancypagestyle{frontpage}{
+}
+\fancypagestyle{page2}[frontpage]{'''
+    tex += r'''
+  \newgeometry{{nomarginpar, top={}, bottom={}, left={}, right={}, headsep={}, footskip={}}}'''.format(tm2,bm2,lm,rm,hs,fs2)
+    tex += r'''
+  \cfoot{\centerline{Page \thepage}}
+  \fancyfootoffset[R]{0pt}% recalculate \headwidth
+}
+\fancypagestyle{datapage}[page2]{'''
+    tex += r'''
+  \newgeometry{{nomarginpar, top={}, bottom={}, left={}, right={}, headsep={}, footskip={}}}'''.format(tm,bm,lm,rm,hs,fs)
+    tex += r'''
+  \rfoot{\textsf{\footnotesize{https://pypi.org/project/sfalmanac/}}}
+} %-----------------------------------'''
+
+    # Note: \DeclareUnicodeCharacter is not compatible with some versions of pdflatex
+    tex += r'''
+\usepackage{xcolor}  % highlight double moon events on same day
+\definecolor{khaki}{rgb}{0.76, 0.69, 0.57}
+\usepackage{multirow}
+\newcommand{\HRule}{\rule{\linewidth}{0.5mm}}
+\usepackage[pdftex]{graphicx}	% for \includegraphics
+\usepackage{tikz}				% for \draw  (load after 'graphicx')
+%\showboxbreadth=50  % use for logging
+%\showboxdepth=50    % use for logging
+%\DeclareUnicodeCharacter{00B0}{\ensuremath{{}^\circ}}
+\setlength\fboxsep{1.5pt}       % ONLY used by \colorbox in ldist_skyfield.py
+\begin{document}'''
+
+    if not config.DPonly:
+        tex += hdrEVnew(first_day,dtp,vsep1,vsep2,df180,df360)
+
+    tex += r'''
+\pagestyle{datapage}  % the default page style for the document'''
+
+    tex += pages(first_day,dtp,strat)
+    tex += r'''
+\end{document}'''
+    return tex
+
+# ===   ===   ===   ===   ===   ===   ===   ===   ===   ===   ===   ===   ===
+# ===   ===   ===   ===   O L D   F O R M A T T I N G   ===   ===   ===   ===
+# ===   ===   ===   ===   ===   ===   ===   ===   ===   ===   ===   ===   ===
+
+def hdrEVold(first_day,dtp,tm1,bm1,lm1,rm1,vsep1,vsep2,df180,df360):
+    # build the front page
+    year = first_day.year
+    mth = first_day.month
+    day = first_day.day
+
+    tex = r'''
+% for the title page only...
+\newgeometry{{nomarginpar, top={}, bottom={}, left={}, right={}}}'''.format(tm1,bm1,lm1,rm1)
+
+    tex += r'''
+    \begin{titlepage}
+    \begin{center}
+    \textsc{\Large Generated using Skyfield}\\
+    \large http://rhodesmill.org/skyfield/\\[0.7cm]'''
+
+    if config.dockerized:   # DOCKER ONLY
+        df180 = "../A4chart0-180_P.pdf"
+        df360 = "../A4chart180-360_P.pdf"
+
+    tex += r'''
+    % TRIM values: left bottom right top
+    \includegraphics[clip, trim=12mm 20cm 12mm 21mm, width=0.92\textwidth]{{{}}}\\[0.3cm]
+    \includegraphics[clip, trim=12mm 20cm 12mm 21mm, width=0.92\textwidth]{{{}}}\\'''.format(df180,df360)
+    
+    tex += r'''[{}]
+    \textsc{{\huge Lunar Distance}}\\[{}]'''.format(vsep1,vsep2)
+
+    if dtp == 0:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(first_day.year)
+    elif dtp == -1:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(first_day.strftime("%B %Y"))
+    elif dtp > 1:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(fmtdates(first_day,first_day+timedelta(days=dtp-1)))
+    else:
+        tex += r'''
+    \HRule \\[0.5cm]
+    {{ \Huge \bfseries {}}}\\[0.2cm]
+    \HRule \\'''.format(fmtdate(first_day))
+
+    tex += r'''
+    \begin{center}\begin{tabular}[t]{rl}
+    \large\emph{Author:} & \large Andrew \textsc{Bauer}\\
+    \end{tabular}\end{center}'''
+
+    tex += r'''
+    {\large \today}
+    \HRule \\[0.2cm]
+    \end{center}
+    \begin{description}[leftmargin=5.5em,style=nextline]\footnotesize
+    \item[Disclaimer:] These are computer generated tables - use them at your own risk. They replicate Lunar Distance algorithms with no guarantee of accuracy. They are intended to encourage the use of sextants, be it as a hobby or as a backup when electronics fail. The author claims no liability for any consequences arising from use of these tables and accompanying charts.
+    \end{description}
+\end{titlepage}
 \restoregeometry    % so it does not affect the rest of the pages'''
 
-    # footskip = distance of footer baseline below body [v2q]
-    alm = alm + r'''
-\setlength{{\footskip}}{{{}}}'''.format(fs)
+    tex += page2()
 
-    alm = alm + pages(first_day,strat,daysnum)
-    alm = alm + '''
+    return tex
+
+def makeLDold(first_day, dtp, strat, df180, df360):
+    # make tables starting from first_day
+    # dtp = 0 if for entire year; = -1 if for entire month; else days to print
+
+    # page size specific parameters
+    if config.pgsz == "A4":
+        # pay attention to the limited page width
+        paper = "a4paper"
+        vsep1 = "1.5cm"
+        vsep2 = "1.0cm"
+        tm1 = "21mm"    # title page...
+        bm1 = "15mm"
+        lm1 = "10mm"
+        rm1 = "10mm"
+        tm = "21mm"     # data pages...
+        bm = "18mm"
+        lm = "16mm"
+        rm = "12mm"
+    else:
+        # pay attention to the limited page height
+        paper = "letterpaper"
+        vsep1 = "0.8cm"
+        vsep2 = "0.7cm"
+        tm1 = "12mm"    # title page...
+        bm1 = "15mm"
+        lm1 = "12mm"
+        rm1 = "12mm"
+        tm = "12mm"   # data pages...
+        bm = "12mm"
+        lm = "16mm"
+        rm = "14mm"
+
+    # default is 'oneside'...
+    tex = r'''\documentclass[10pt, {}]{{report}}'''.format(paper)
+
+    tex += r'''
+%\usepackage[utf8]{inputenc}
+\usepackage[english]{babel}
+\usepackage{fontenc}
+\usepackage{enumitem} % used to customize the {description} environment'''
+
+    # to troubleshoot add "showframe, verbose," below:
+    tex += r'''
+\usepackage[nomarginpar, top={}, bottom={}, left={}, right={}]{{geometry}}'''.format(tm,bm,lm,rm)
+
+    # Note: \DeclareUnicodeCharacter is not compatible with some versions of pdflatex
+    tex += r'''
+\usepackage{xcolor}  % highlight double moon events on same day
+\definecolor{khaki}{rgb}{0.76, 0.69, 0.57}
+\usepackage{multirow}
+\newcommand{\HRule}{\rule{\linewidth}{0.5mm}}
+\setlength{\footskip}{15pt}
+\usepackage[pdftex]{graphicx}	% for \includegraphics
+\usepackage{tikz}				% for \draw  (load after 'graphicx')
+%\showboxbreadth=50  % use for logging
+%\showboxdepth=50    % use for logging
+%\DeclareUnicodeCharacter{00B0}{\ensuremath{{}^\circ}}
+\setlength\fboxsep{1.5pt}       % ONLY used by \colorbox in ldist_skyfield.py
+\begin{document}'''
+
+    if not config.DPonly:
+        tex += hdrEVold(first_day,dtp,tm1,bm1,lm1,rm1,vsep1,vsep2,df180,df360)
+
+    tex += pages(first_day,dtp,strat)
+    tex += r'''
 \end{document}'''
-    return alm
\ No newline at end of file
+    return tex
\ No newline at end of file
